/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.device.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.device.domain.DmDevice;
import me.zhengjie.modules.dm.device.service.DmDeviceService;
import me.zhengjie.modules.dm.device.service.dto.DmDeviceDto;
import me.zhengjie.modules.dm.device.service.dto.DmDeviceQueryCriteria;
import me.zhengjie.modules.dm.printingQueue.domain.DmPrintingQueue;
import me.zhengjie.modules.dm.printingQueue.service.DmPrintingQueueService;
import me.zhengjie.modules.dm.printingQueue.service.dto.DmPrintingQueueQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import java.util.Map;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author xwx
* @date 2021-03-12
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "dmDevice管理")
@RequestMapping("/api/dmDevice")
public class DmDeviceController {

    private final DmDeviceService dmDeviceService;
    private final DmPrintingQueueService dmPrintingQueueService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmDevice:list')")
    public void download(HttpServletResponse response, DmDeviceQueryCriteria criteria) throws IOException {
        dmDeviceService.download(dmDeviceService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询dmDevice")
    @ApiOperation("查询dmDevice")
    @PreAuthorize("@el.check('dmDevice:list')")
    public ResponseEntity<Object>  query(DmDeviceQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmDeviceService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增dmDevice")
    @ApiOperation("新增dmDevice")
    @PreAuthorize("@el.check('dmDevice:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmDevice resources){
        return new ResponseEntity<>(dmDeviceService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改dmDevice")
    @ApiOperation("修改dmDevice")
    @PreAuthorize("@el.check('dmDevice:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmDevice resources){
        dmDeviceService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除dmDevice")
    @ApiOperation("删除dmDevice")
    @PreAuthorize("@el.check('dmDevice:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        Map<String,Object> map=dmDeviceService.deleteAll(ids);
        return new ResponseEntity<>(map,HttpStatus.OK);
    }

    @GetMapping(value = "/all")
    @Log("查询dmDevice")
    @ApiOperation("查询dmDevice")
    public ResponseEntity<Object> all(){
        return new ResponseEntity<>(dmDeviceService.queryAll(),HttpStatus.OK);
    }

    @GetMapping(value = "/selectList")
    @Log("查询选择列表")
    @ApiOperation("查询选择列表")
    public ResponseEntity<Object> selectList(DmDeviceQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmDeviceService.selectList(criteria,pageable),HttpStatus.OK);
    }

    @GetMapping(value = "info")
    @Log("查询dmDevice")
    @ApiOperation("查询dmDevice")
    public ResponseEntity<Object> info(Long id){
        if(id == null ){
            return null;
        }
        return new ResponseEntity<>(dmDeviceService.findInfo(id),HttpStatus.OK);
    }

    @GetMapping(value = "firstDevice")
    @Log("首页设备信息")
    @ApiOperation("首页设备信息")
    public ResponseEntity<Object> firstDevice(){
        return new ResponseEntity<>(dmDeviceService.firstDevice(),HttpStatus.OK);
    }

    @GetMapping(value = "/yunDeviceRequest")
    @Log("查询设备运行趋势")
    @ApiOperation("查询设备运行趋势")
    public ResponseEntity<Object> yunDeviceRequest(){
        return new ResponseEntity<>(dmDeviceService.YunDeviceRequest(),HttpStatus.OK);
    }

    @GetMapping(value = "/selectDevice")
    @Log("查询设备打印任务")
    @ApiOperation("查询设备打印任务")
    public ResponseEntity<Object> selectDevice(String buildStateDate,  String buildEndDate, Pageable pageable){
        return new ResponseEntity<>(dmDeviceService.selectDevice(buildStateDate,buildEndDate,pageable),HttpStatus.OK);
    }
}
