/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.printingApproval.service;

import me.zhengjie.base.BaseResult;
import me.zhengjie.modules.dm.printingApproval.domain.DmPrintingApproval;
import me.zhengjie.modules.dm.printingApproval.service.dto.DmPrintingApprovalDto;
import me.zhengjie.modules.dm.printingApproval.service.dto.DmPrintingApprovalQueryCriteria;
import org.springframework.data.domain.Pageable;
import java.util.Map;
import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author xwx
* @date 2021-03-17
**/
public interface DmPrintingApprovalService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmPrintingApprovalQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmPrintingApprovalDto>
    */
    List<DmPrintingApprovalDto> queryAll(DmPrintingApprovalQueryCriteria criteria);

    /**
     * 根据ID查询
     * @param id ID
     * @return DmPrintingApprovalDto
     */
    DmPrintingApprovalDto findById(Long id);

    /**
    * 创建
    * @param resources /
    * @return DmPrintingApprovalDto
    */
    DmPrintingApprovalDto create(DmPrintingApproval resources);

    /**
    * 编辑
    * @param resources /
    */
    void update(DmPrintingApproval resources);

    /**
    * 多选删除
    * @param ids /
    */
    void deleteAll(Long[] ids);


    void deleteByRequestId(Long requestId);

    /**  TODO 审批
     * */
    BaseResult approval(DmPrintingApproval resources);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmPrintingApprovalDto> all, HttpServletResponse response) throws IOException;

    /**
     * TODO 查询最大的code
     * @return
     */
    String getMaxCode();

    /**
     * todo 修改审批人
     */
    void editApprovalUser(String approvalUser,Long requestId);

    /**
     * todo 将历史审批信息设置为旧信息
     * @param requestId
     */
    void changeOld(Long requestId);

    /**
     * todo 初始化审批信息
     * @param requestId
     */
    void reset(Long requestId);

    /**
     * todo 初始化审批任务
     */
    void resetTask(Long requestId,String approvalUser,String taskId,String procInstId);
}