/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.printingQueue.rest;

import me.zhengjie.annotation.Log;
import me.zhengjie.modules.dm.printingQueue.domain.DmPrintingQueue;
import me.zhengjie.modules.dm.printingQueue.service.DmPrintingQueueService;
import me.zhengjie.modules.dm.printingQueue.service.dto.DmPrintingQueueQueryCriteria;
import org.springframework.data.domain.Pageable;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.*;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @author xwx
* @date 2021-03-17
**/
@RestController
@RequiredArgsConstructor
@Api(tags = "dmPrintingQueue管理")
@RequestMapping("/api/dmPrintingQueue")
public class DmPrintingQueueController {

    private final DmPrintingQueueService dmPrintingQueueService;

    @Log("导出数据")
    @ApiOperation("导出数据")
    @GetMapping(value = "/download")
    @PreAuthorize("@el.check('dmPrintingQueue:list')")
    public void download(HttpServletResponse response, DmPrintingQueueQueryCriteria criteria) throws IOException {
        dmPrintingQueueService.download(dmPrintingQueueService.queryAll(criteria), response);
    }

    @GetMapping
    @Log("查询dmPrintingQueue")
    @ApiOperation("查询dmPrintingQueue")
    @PreAuthorize("@el.check('dmPrintingQueue:list')")
    public ResponseEntity<Object> query(DmPrintingQueueQueryCriteria criteria, Pageable pageable){
        return new ResponseEntity<>(dmPrintingQueueService.queryAll(criteria,pageable),HttpStatus.OK);
    }

    @PostMapping
    @Log("新增dmPrintingQueue")
    @ApiOperation("新增dmPrintingQueue")
    @PreAuthorize("@el.check('dmPrintingQueue:add')")
    public ResponseEntity<Object> create(@Validated @RequestBody DmPrintingQueue resources){
        return new ResponseEntity<>(dmPrintingQueueService.create(resources),HttpStatus.CREATED);
    }

    @PutMapping
    @Log("修改dmPrintingQueue")
    @ApiOperation("修改dmPrintingQueue")
    @PreAuthorize("@el.check('dmPrintingQueue:edit')")
    public ResponseEntity<Object> update(@Validated @RequestBody DmPrintingQueue resources){
        dmPrintingQueueService.update(resources);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Log("删除dmPrintingQueue")
    @ApiOperation("删除dmPrintingQueue")
    @PreAuthorize("@el.check('dmPrintingQueue:del')")
    @DeleteMapping
    public ResponseEntity<Object> delete(@RequestBody Long[] ids) {
        dmPrintingQueueService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}