/*
*  Copyright 2019-2020 Zheng Jie
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*  you may not use this file except in compliance with the License.
*  You may obtain a copy of the License at
*
*  http://www.apache.org/licenses/LICENSE-2.0
*
*  Unless required by applicable law or agreed to in writing, software
*  distributed under the License is distributed on an "AS IS" BASIS,
*  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*  See the License for the specific language governing permissions and
*  limitations under the License.
*/
package me.zhengjie.modules.dm.printingRequest.service;

import me.zhengjie.modules.dm.printingRequest.domain.DmPrintingRequest;
import me.zhengjie.modules.dm.printingRequest.service.dto.DmPrintingRequestDto;
import me.zhengjie.modules.dm.printingRequest.service.dto.DmPrintingRequestQueryCriteria;
import me.zhengjie.modules.dm.printingRequest.service.dto.FindDmPrintingRequestParam;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import java.util.Map;
import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;

/**
* @website https://el-admin.vip
* @description 服务接口
* @author xwx
* @date 2021-03-15
**/
public interface DmPrintingRequestService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(DmPrintingRequestQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<DmPrintingRequestDto>
    */
    List<DmPrintingRequestDto> queryAll(DmPrintingRequestQueryCriteria criteria);

    /**
     * 根据ID查询
     * @param id ID
     * @return DmPrintingRequestDto
     */
    DmPrintingRequestDto findById(Long id);

    /**
    * 创建
    * @param resources /
    * @return DmPrintingRequestDto
    */
    DmPrintingRequestDto create(DmPrintingRequest resources) throws IOException;

    /**
    * 编辑
    * @param resources /
    */
    void update(DmPrintingRequest resources,String ... status);

    /**
     * 选择审批人
     * @param resources /
     * @return DmPrintingRequestDto
     */
    void addApprovalUser(DmPrintingRequest resources);

    /**
     * 撤回
     * @param resources /
     * @return DmPrintingRequestDto
     */
    void recallApprovalUser(DmPrintingRequest resources);

    void message(DmPrintingRequestDto dmPrintingRequest,String taskId,String sendContent);

    /**
    * 多选删除
    * @param ids /
    */
    Map<String,Object> deleteAll(Long[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<DmPrintingRequestDto> all, HttpServletResponse response) throws IOException;

    DmPrintingRequestDto findInfo(Long id);

    String getMaxCode();

    void updatePringTask(DmPrintingRequest resources);

    DmPrintingRequest toEntity(DmPrintingRequestDto dto);

    Page<Map<String,Object>> findByRequestUser(String status, String printingRequest, String requestUser, Pageable pageable);

    Map<String,Object> findMyWy(FindDmPrintingRequestParam findDmPrintingRequestParam, Pageable pageable);

    /**
     * 根据ID查询
     * @param deviceId ID
     * @return
     */
    List<Map<String,Object>> requestListByDeviceId(Long deviceId);

    Long findRequestCount();

    List<Map<String,Object>> findRequestInfo();
}
